from setuptools import setup
setup(name='hjn',
      version='0.1.21',
      description='hjn in CMA',
      url=' ',
      author='hjn',
      author_email='hujunnan2@126.com',
      license='MIT',
      packages=['hjn'],
      zip_safe=False)
